from __future__ import division
from numpy import array
import numpy.linalg as LA
from numpy import inner
from Map import Map
import PoIs
import sys

relevant_radius = 100

EPS = 0.0000001
simulationObject = None


def simpleFall(person, timestep, _map, destination):
    """
    wunschrichtungs vektor
    """
    d = array([0.0, 0.0])
    try:
        destinationPos = destination.getPosition()  # destination is person oder PoI
    except:
        destinationPos = destination

    if destinationPos is not None:
        person.sp = _map.getShortestPath(person.getPosition(), destinationPos)
        dist = LA.norm(person.getPosition() - destinationPos)
        if dist > person.destReachedThreshold:
            vektorToDestination = (person.sp[0] - person.getPosition())
#            vektorToDestination= _map.dirToExit(destination)
            accel = timestep * vektorToDestination * 0.1 * pow(dist/10 + EPS, 2) #(0.1 * min(100, dist))**2
            return d + accel
        elif dist < person.destReachedThreshold:
            print(type(destination))
            if type(destination) is PoIs.Exit:
                PoIs.Exit(destination).swallowedPersons = person
                simulationObject.personsToRemove.append(person)
        return d
    return d * 0.2

def getDesire(person, timestep, _map, next_targets):
    #in prozess
    """
    wunschrichtungs vektor
    """

    d = array([0.0, 0.0])

    
    # Falls wir die zwischenergebnisse speichern wollen
    # person.sp = _map.getShortestPath(person.getPosition(), destinationPos)

    if next_targets is not None:
        next_target = next_targets[0]
        dist = LA.norm(person.getPosition() - next_target)
        if dist > person.destReachedThreshold:# bin ich nahe am ziel
            vektorToDestination = (person.sp[0] - person.getPosition())
            accel = timestep * vektorToDestination * 0.1 * pow(dist/10 + EPS, 2) #(0.1 * min(100, dist))**2
            return d + accel
        elif dist < person.destReachedThreshold:
            if type(destination) is PoIs.Exit:
                PoIs.Exit(destination).swallowedPersons = person
                simulationObject.personsToRemove.append(person)
        return d
    return d * 0.2

def getMovement(person, timestep, nearbyPeople, _map):
    """
    berechnet den bewegungsvektor anhand von den anderen funktionen
    """
    d = array([0.0, 0.0])
    d += person.last_move
    if(len(_map.getExits())>0):
        d += simpleFall(person, timestep, _map, _map.getExits()[0])
    d += impactHindernisse(person, _map)
    d += impactPersonen(person, nearbyPeople)

    speed = LA.norm(d) / timestep
    factor = min(speed, person.maxSpeed) / speed
    return d * factor

def impactPersonen_cas(person, nearbyPeople):
    """
    kraftabstosung einer person

    Berechnet fuer eine Person und nahe Personen, 
    die richtung in welche die person von den nahen Personen gedrueckt wird wird
    """
    impact = array([0.0,0.0])
    for neighbor in nearbyPeople:
        dist = person.getPosition() - neighbor.getPosition()
        force = dist / (pow(LA.norm(dist), 3) + EPS)
        impact += force
    return impact

def impactPersonen2(person, nearbyPeople):
    d = array([0.0,0.0])
    

def impactPersonen(person, nearbyPeople):
    """
    kraftabstosung einer person
    """
    d = array([0.0,0.0])
    for neighbor in nearbyPeople:
        dist = person.getPosition() - neighbor.getPosition()
        force = dist / pow(LA.norm(dist) + EPS, 3)
        if inner(d, force) > 0:
            force = force * 0.01
        d += force
    return d


def impactHindernisse_cas(person, _map):
    """
    kraftabstosung vom hindernis
    """
    impact = array([0.0,0.0])
    for obstacle in _map.getObstaclesInRange(relevant_radius, person.getPosition()):
        lot = (-1)*obstacle.getLot(person.getPosition())
        force = lot / (pow(LA.norm(lot), 3)+EPS)
        impact += force
    return impact


def impactHindernisse(person, _map):
    """
    kraftabstosung vom hindernis
    """
    d = array([0.0,0.0])
    for obstacle in _map.getObstaclesInRange(relevant_radius, person.getPosition()):
        lot = obstacle.getLot(person.getPosition())
        dist = LA.norm(lot) / 3
        force = -lot / pow(dist + EPS, 3)
        if inner(d, force) > 0:
            force = force * .01
        d += force
    return d
