# finds all shortest paths to exit for some instace
from collections import defaultdict
from numpy import array
from numpy import linalg as LA

        
class Graph:
    def __init__(self):
        self.nodes = set()
        self.edges = defaultdict(list)
        self.distances = {}

    def add_node(self, value):
        self.nodes.add(value)

    def add_edge(self, from_node, to_node, distance):
        self.edges[from_node].append(to_node)
        self.edges[to_node].append(from_node)
        self.distances[(from_node, to_node)] = distance
        self.distances[(to_node, from_node)] = distance


def dijkstra(graph, initial):
    visited = {initial: 0}
    path = {}

    nodes = set(graph.nodes)
    
    while nodes: 
        min_node = None
        for node in nodes:
            if node in visited:
                if min_node is None:
                    min_node = node
                elif visited[node] < visited[min_node]:
                        min_node = node

        if min_node is None:
            break

        nodes.remove(min_node)
        current_weight = visited[min_node]
    
        for edge in graph.edges[min_node]:
            weight = current_weight + graph.distances[(min_node, edge)]
            if edge not in visited or weight < visited[edge]:
                visited[edge] = weight
                path[edge] = min_node
    return visited, path

def getShortestPath(map_, start, target):
    """
    gibt zurueck: liste mit [path{id -->id}, map[id -->point], edge(id,id)-list].
    id von start ist 0, id vom target ist 1
    """
    #graphen basteln
    edgeList= []
    for edge in map_.edgeList:
        edgeList+= [edge+2]
    poi=[start, target] + map_.idxToPoint

    obstacles= map_.getObstacles()
    n= len(poi)
    g= Graph()
    for i in range(n):
        g.add_node(i)
    for i in [0, 1]:
        for j in range(i+1,n):
            line= array((poi[i], poi[j]))
            free= True
            for obs in obstacles:
                if obs.intersects(line, array([True , False])):
                    free= False
            if free:
                edgeList+=[array((i,j))]
                
    for edge in edgeList:
        g.add_edge(edge[0], edge[1], LA.norm(poi[edge[0]]-poi[edge[1]]))
        
    dijk= dijkstra(g, 1)
    path= dijk[1]
    distances= dijk[0]
        
    return [path, poi, edgeList, distances]