from Obstacle import Obstacle
from numpy import array
from numpy import linalg as LA
import numpy as np
import pickle
from Dijkstra import getShortestPath as computeSP
from math import ceil, floor

EPS=0.00001

class Map(object):
    """represents a room or building"""

    comftDist= 20 # distance from obstacle corner to where we want to walk
    
    edgeList=[] #contains the indices, not the coordinates
    idxToPoint=[] #contains the coordinates
    idxToComftPoint= [] # contains the coordinates a bit away from the obstacles
    
    accuracy= 20
    #{exit}{x][y][[pathToExit]]
    toExit={}
    
    sp={}

    def __init__(self):
        self.width = 0
        self.height = 0
        self.obstacles = []
        self.exits = []

    def precomputeSP(self):
        self.refreshPaths()
        start= array((0,0)) #just a buffer
        for ex in self.exits :
            dijk= computeSP(self, start, ex.position)
            #return [path, poi, edgeList, distances]
            path= dijk[0]
            dist= dijk[3]
            self.sp[ex]= [path, dist]
            
        w= ceil(self.width/self.accuracy)
        h= ceil(self.height/self.accuracy)
        dirs= array( ((1, 1), (1, 0), (1, -1), (0, -1), (-1, -1), (-1, 0), (-1, 1), (0, 1)) )
        steps= 4
        for ex in self.exits :
            ar= array(np.zeros((w,h)), dtype=object)
            for x in range(w) :
                print("countdown:", w-x)
                for y in range(h) :
                    #scatter some points over the grid square
                    mid= self.accuracy* array((x+.5,y+.5))
                    testpoints= [mid]
                    for i in range(1, steps):
                        for j in range(8) :
                            testpoints+=[mid+dirs[j]*self.accuracy*.5*i/steps]
                    # finde den mittigsten punkt, der in keinem polygon liegt
                    mostCentral= None
                    for tp in testpoints:
                        if self.pointIsAvailable(tp) :
                            mostCentral= tp
                            break
                    # compute the shortest path to exit
                    if not mostCentral is None:
                        ar[x,y]= self.getShortestPath(mostCentral, ex.position)
                    else:
                        ar[x,y]= -1  
            self.toExit[ex]= ar
            
    
    def dirToExit(self, pos, ex):
        """gibt die Richtung, die auf dem kuerzesten weg zum naechsten Ausgang liegt"""
        raise NotImplementedError

    @staticmethod
    def readFromFile(filename):
        with open(filename, 'rb') as input:
            loadedMap = pickle.load(input)
            loadedMap.refreshPaths()
            return loadedMap

    def pointIsAvailable(self, pos):
        """returns, whether a person can be at that point"""
        res= True
        for obs in self.obstacles:
            if obs.isInside(pos):
                res= False
        return res
        
    def lineIsAvailable(self, start, target, harmless= array((False, False))):
        isAvailable= True
        for obs in self.obstacles :
            if obs.intersects(array((start, target)), harmless):
                isAvailable= False
        
        return isAvailable
    
    def getObstaclesInRange(self, r, point):
        enviroment=[]
        for obs in self.obstacles :
            if obs.distanceTo(point) < r:
                enviroment+=[obs]
        return enviroment

    def addObstacle(self, obstacle):
        """
        fuegt ein einzelnes Obstacle zur Hindernisliste hinzu
        """
        self.obstacles.append(obstacle)
        self.refreshPaths()
        
    def addObstacles(self, obstacles):
        """
        erwartet eine liste von Obstacle, die dann hinzugefuegt wird
        """
        self.obstacles+= obstacles
        self.refreshPaths()
        
    def getShortestPath(self, start, target):
        """
        gibt liste der zu besuchenden punkte zurueck. Der naechste punkt steht am anfang der liste
        """
        #first check, if the straight line is available
        if self.lineIsAvailable(start, target) :
            return [target]
            
        #check if the target is an exit            
        found= False
        path={}
        for ex in self.exits :
            if LA.norm(target-ex.position) < EPS:
                if ex in self.toExit.keys():
                    x= floor(start[0]/self.accuracy)
                    y= floor(start[1]/self.accuracy)
                    if not self.toExit[ex][x,y] == -1:
                        return self.toExit[ex][x,y]                
                if ex in self.sp.keys() :
                    found= True
                    path= self.sp[ex][0]
                    dists= self.sp[ex][1]
                    # find the best corner
                    min_dist= 9999999999
                    for idx in dists.keys():
                        if idx != 0 and idx != 1:
                            coords= self.idxToPoint[idx-2]
                            if self.lineIsAvailable(start, coords, array((True, False))) :
                                dist= dists[idx]+LA.norm(start-coords)
                                if dist < min_dist:
                                    min_dist= dist
                                    path[0]= idx
            
        if not found :
            dijkstra= computeSP(self, start, target)
            path= dijkstra[0]
            
        toVisit=[]
        idx= 0 #das ist der startpunkt
        try:
            while idx != 1:
                idx= path[idx]
                if idx==1:
                    toVisit+=[target]
                else:
                    toVisit+=[self.idxToComftPoint[idx-2]]
        except:
            #if anything fails, just run towards the target
            toVisit=[target]
        return toVisit
        
    def refreshPaths(self):
        """
        vorberechnet alle geraden zwischen eckpunkten, die begehbar sind 
        """
        self.idxToComftPoint=[]
        self.idxToPoint=[]
        self.edgeList=[]
        for obs in self.obstacles:
            corners= obs.getCorners()
            c= len(corners)
            for i in range(c):
                p= corners[i]
                self.idxToPoint+=[p]
                prevP= corners[(i-1)%c]
                nextP= corners[(i+1)%c]
                v1= p-prevP
                v2= p-nextP
                v1= v1/LA.norm(v1)
                v2= v2/LA.norm(v2)
                v=v1+v2
                v= v*self.comftDist/LA.norm(v)
                if obs.isInside(p+EPS*v):
                    v= -v
                self.idxToComftPoint+=[p+v]
        n= len(self.idxToPoint)
        for i in range(n):
            for j in range(i+1,n):
                line= array((self.idxToPoint[i], self.idxToPoint[j]))
                free= True
                for obs in self.obstacles:
                    if obs.intersects(line):
                        free= False
                if free:
                    self.edgeList+=[array((i,j))]

    def getObstacles(self):
        return self.obstacles

    def addExit(self, exit_):
        self.exits.append(exit_)

    def getExits(self):
        return self.exits

    def becomeTestInstance(self):
        self.width= 300
        self.height=500
        ramp= Obstacle( [array((-100,-100)),array((100,-100)),array((100,-70))])
        self.obstacles+=[ramp]

    def setWidth(self, width):
        self.width = width

    def setHeight(self, height):
        self.height = height