from shapely.geometry import Point
from shapely.geometry import Polygon
from numpy import array, cos, sin, pi
import numpy.linalg as LA

EPS= 0.00000001

def mul(b, A):
    return array((b[0]*A[0,0]+b[1]*A[0,1], b[0]*A[1,0]+b[1]*A[1,1]))

class Obstacle(object):
    EPS= 0.00001
    def __init__(self, pointList):
        self.polygon= Polygon(pointList)
        #ugly, but the easiest way until i found out the correct function in shapely.polygon
        self.pointList= pointList
        
    def getCorners(self):
        return self.pointList

    def distanceTo(self, p):
        point= Point(p)
        return self.polygon.distance(point)

    def isInside(self, p):
        point= Point(p)
        return self.polygon.contains(point)

    def nearestEdge(self, p):
        corners= self.pointList
        nearest= array([[0,0],[0,0]])
        mini=9999999999
        n= len(corners)
        for i in range(n):
            head= corners[i]
            tail= corners[(i+1)%n]
            line= Polygon([head,tail,head])
            dist= line.distance(Point(p))
            if dist < mini:
                mini= dist
                nearest= array([head,tail])
        return nearest

    def getLot(self, p):
        """segment FROM the point to its nearest point on the polygon"""
        edge= self.nearestEdge(p)
        s, t= edge[0], edge[1]
        mid=.5*(s+t)
        orth= array((s[1]-t[1], -(s[0]-t[0])))
        orth=orth/LA.norm(orth)
        test= mid+ EPS*orth
        if self.isInside(test):
            return orth*self.distanceTo(p)
        else:
            return -orth*self.distanceTo(p)

    def intersects(self, line, harmless=array([False, False])):
        """berechnet, ob die linie das polygon schneidet. line sollte gegeben sein als 2x2-array"""
        step= 3
        start= line[0]
        end= line[1]
        starts=[]
        ends=[]
        
        for i in range(step):
            a= 2*pi*i/step #winkel
            A=array([[cos(a),-sin(a)],[sin(a),cos(a)]]) #drehmatrix
            starts+= [start + 10*self.EPS* mul(array((1,0)),A)]
            ends+= [end + 10*self.EPS*mul(array((1,0)),A)]
        
        if harmless[0] :
            starts= [start]
            
        if harmless[1] :
            ends= [end]
        
        distances=[]        
        for s in starts:
            for e in ends:
                linePolygon= Polygon([s,e,s])
                distances+=[linePolygon.distance(self.polygon)]

        if max(distances) < self.EPS :
            return True
        return False
