import pygame
from pygame.locals import *
import numpy as np
from numpy import array
from Map import Map
from Obstacle import Obstacle
from PoIs import Exit
import pickle


def drawPolygon(pointList, color):
    pygame.draw.polygon(window, color, pointList, 2);


def invalidate():
    global invalidated
    invalidated = True


def getPolysAsListWithNP_ArrayonLowLevel():
    print(polygons)
    output = []
    for poly in polygons:
        mypoly = []
        for point in poly:
            npPoint = np.asarray(point)
            mypoly.append(npPoint)
        output.append(mypoly)
    return output


def exportAsMap(filename):
    myMap = Map()
    polys = getPolysAsListWithNP_ArrayonLowLevel()
    for poly in polys:
        myMap.addObstacle(Obstacle(poly))

    for exit_ in exits:
        myMap.addExit(exit_)

    myMap.setWidth(pygame.display.Info().current_w)
    myMap.setHeight(pygame.display.Info().current_h)

    with open(filename, 'wb') as output:
        pickle.dump(myMap, output, pickle.HIGHEST_PROTOCOL)


polygons = []
activePoly = []
exits = []
invalidated = False
clock = pygame.time.Clock()
pygame.init()
window = pygame.display.set_mode((1200, 1200), 0, 16)
window.fill(([255] * 3))
pygame.display.set_caption("Simple Scene Editor")
running = True
invalidate()

while running is True:
    for event in pygame.event.get():
        if event.type is pygame.QUIT:
            running = False

        elif event.type is pygame.KEYDOWN:
            if event.key is pygame.K_q:  # quit
                running = False
            elif event.key is pygame.K_c:  # complete Poly
                if (len(activePoly) >= 3):
                    polygons.append(activePoly)
                if (len(activePoly) > 0):
                    activePoly = []
                    invalidate()
            elif event.key is pygame.K_p:  # create Exit
                exits.append(Exit(position=array(pygame.mouse.get_pos())))
                invalidate()
            elif event.key is pygame.K_e:
                print("exporting")
                exportAsMap('MapTestPickle.pkl')
            elif False and event.key is pygame.K_g:  # print list
                print(getPolysAsListWithNP_ArrayonLowLevel())
            elif False and event.key is pygame.K_t:  # Test for loading Map Object from File
                newMap = Map.readFromFile('MapTestPickle.pkl')
                print(vars(newMap))
                for obst in newMap.getObstacles():
                    print(vars(obst))

        elif event.type is pygame.MOUSEBUTTONDOWN:
            activePoly.append(pygame.mouse.get_pos())
            invalidate()
        elif event.type is pygame.MOUSEMOTION:
            if len(activePoly) > 0:
                invalidate()

    if invalidated:
        # draw Polygons
        window.fill([255] * 3)

        helpText = "help: Q -> quit; C -> complete poly; e -> export"
        window.blit(pygame.font.SysFont("monospace", 15).render(helpText, 1, (0, 0, 200)), (0, 0))

        for poly in polygons:
            drawPolygon(poly, (0, 0, 0))
        if len(activePoly) >= 1:
            drawPolygon(activePoly + [pygame.mouse.get_pos()], (250, 100, 0))

        for exit in exits:
            pygame.draw.circle(window,(0,200,0), exit.getPosition(),10,0)
        invalidated = False
    pygame.display.update()
    clock.tick(10)

# end
pygame.quit()
quit()
